/****************************************************************************
**
** $Id: CustomizeShortcutsDialogImpl.cpp,v 1.15 2004/01/11 21:55:21 riemer Exp $
**
** Copyright (C) 1999-2004 The LinCVS development team.
**    Tilo Riemer <riemer@lincvs.org>
**    Falk Brettschneider <gigafalk@yahoo.com>
**    Frank Hemer <frank@hemer.org>
**    Wim Delvaux <wim.delvaux@chello.be>
**    Jose Hernandez <joseh@tesco.net>
**    Helmut Koll <HelmutKoll@web.de>
**    Tom Mishima <tmishima@mail.at-m.or.jp>
**    Joerg Preiss <auba@auba.de>
**    Sven Trogisch <trogisch@iapp.de>
**
**
**----------------------------------------------------------------------------
**
**----------------------------------------------------------------------------
**
** LinCVS is available under two different licenses:
**
** If LinCVS is linked against the GPLed version of Qt 
** LinCVS is released under the terms of GPL also.
**
** If LinCVS is linked against a nonGPLed version of Qt 
** LinCVS is released under the terms of the 
** LinCVS License for non-Unix platforms (LLNU)
**
**
** LinCVS License for non-Unix platforms (LLNU):
**
** Redistribution and use in binary form, without modification, 
** are permitted provided that the following conditions are met:
**
** 1. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 2. It is not permitted to distribute the binary package under a name
**    different than LinCVS.
** 3. The name of the authors may not be used to endorse or promote
**    products derived from this software without specific prior written
**    permission.
** 4. The source code is the creative property of the authors.
**    Extensions and development under the terms of the Gnu Public License
**    are limited to the Unix platform. Any distribution or compilation of 
**    the source code against libraries licensed other than gpl requires 
**    the written permission of the authors.
**
**
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR 
** IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
** ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY 
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
** GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
** WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
**
**
** LinCVS License for Unix platforms:
**
** This program is free software; you can redistribute it and/or modify 
** it under the terms of the GNU General Public License as published by 
** the Free Software Foundation; either version 2 of the License, or 
** (at your  option) any later version.  This program is distributed in 
** the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
** even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
** PARTICULAR PURPOSE.
**
** See the GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**
*****************************************************************************/

#include "config.h"

#include <qobjectlist.h>
#include <qregexp.h>
#include <qmessagebox.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qpushbutton.h>
#include <qwhatsthis.h>

#include "CustomizeShortcutsDialogImpl.h"
#include "globals.h"
#include "GrabKeyboardImpl.h"
#include "lincvs.h"

//List
//NoneRButton
//CustomRButton

namespace Menu {
  Type& operator++(Type& t) {
    switch (t) {
      case PROJECT: return t=DIR;
      case DIR: return t=FILE;
      case FILE: return t=OPTIONS;
      case OPTIONS: return t=HELP;
      default: return t;
    }
  }
  Type operator++(Type& t,int) {
    Type h = t;
    ++t;
    return h;
  }
};

CustomizeShortcutsDialogImpl::CustomizeShortcutsDialogImpl(const QIconSet &whatsThisIconSet,
							   QMainWindow * parent) 
  : CustomizeShortcutsDialog(parent, "ShortCuts", true, 
                                        Qt::WStyle_SysMenu | Qt::WStyle_MinMax | 
                                        Qt::WStyle_DialogBorder | Qt::WType_Dialog |
                                        Qt::WStyle_ContextHelp), 
    m_parent(parent) 
{
  m_pWhatsThis->setIconSet(whatsThisIconSet);
  m_pWhatsThis->setMaximumWidth(m_pWhatsThis->height());

  keyMap.clear();
  RadioButtonGroup->setEnabled(FALSE);
  fillList();
}

CustomizeShortcutsDialogImpl::~CustomizeShortcutsDialogImpl() {
}

CustomizeShortcutsDialogImpl::ListItem * CustomizeShortcutsDialogImpl::dynamic_cast_listItem(QListViewItem * item) {
  if (item->rtti() == 1000) {
    return (ListItem*)item;
  } else {
    return NULL;
  }
}

void CustomizeShortcutsDialogImpl::showGrabKeyboard() {
  
  if ( !CustomRButton->isChecked()) return;

  ListItem * listItem = dynamic_cast_listItem(List->currentItem());
  int accel = 0;
  if (listItem) accel = listItem->getAccel();

  GrabKeyboardImpl * dlg = new GrabKeyboardImpl(*(m_pWhatsThis->iconSet()), this, accel);

  if (dlg->exec()) {
    QKeySequence ks(dlg->getKeySequence());
    setShortcut(ks);
  }
  delete dlg;

}

void CustomizeShortcutsDialogImpl::applyClicked() {
  KEYBOARDSHORTCUTS = "";
  QValueList<int> values = keyMap.values();
  MenuItem * item = MenuItem::first();
  while (item) {
    if ( values.contains(item->getInstanceNum()) ) {
      QMap<int,int>::Iterator it;
      for ( it = keyMap.begin(); it != keyMap.end(); ++it ) {
	if (it.data() == item->getInstanceNum()) {
	  MenuItem::setAccel(item,it.key());
	  KEYBOARDSHORTCUTS += QString::number(it.data())+","+QString::number(it.key())+":";
	  break;
	}
      }
    } else {
      MenuItem::setAccel(item,0);
    }
    item = item->nextItem();
  }
}

void CustomizeShortcutsDialogImpl::selected(QListViewItem * item) {

  RadioButtonGroup->setEnabled(item->isSelected());
  if (!item->isSelected()) NoneRButton->setChecked(TRUE);

  ListItem * listItem = dynamic_cast_listItem(item);
  if (listItem) {
    int accel = listItem->getAccel();
    if (accel==0) NoneRButton->setChecked(TRUE);
    else CustomRButton->setChecked(TRUE);
    QKeySequence ks(accel);
    emit setText(QString(ks));
  } else emit setText("");
  EditButton->setEnabled(CustomRButton->isChecked());
}

void CustomizeShortcutsDialogImpl::customizable(int a) {
  EditButton->setEnabled(a == 1);
}

void CustomizeShortcutsDialogImpl::resetAccel() {
  ListItem * listItem = dynamic_cast_listItem(List->currentItem());
  if (listItem && (listItem->getAccel() != 0) ) {
    QKeySequence seq(0);
    setShortcut(seq);
  }
}

void CustomizeShortcutsDialogImpl::fillList() {

  QMenuBar * menuBar = m_parent->menuBar();
  int index = 0;
  int id = -1;
  QListViewItem * rootItem = NULL;
  Menu::Type type = Menu::PROJECT; 
  while ( (id = menuBar->idAt(index++)) > -1) {
    QMenuItem * topItem = menuBar->findItem(id);
    if (topItem) {
      QString txt = topItem->text();
      txt.replace( QRegExp("&"),"");
      rootItem = new QListViewItem(List,txt);
      rootItem->setSelectable(FALSE);

      MenuItem * item = MenuItem::first();
      while (item) {
	if ( (item->menu() & type) && item->slot() ) {
	  int accel = item->accel();
	  QString txt = item->caption();
	  MenuItem * parent = item->getParent();
	  while (parent) {
	    txt = parent->caption()+" "+txt;
	    parent = parent->getParent();
	  }
	  new ListItem(rootItem,txt,QString(QKeySequence(accel)),item );
	  if (accel != 0) {
	    keyMap.insert(accel,item->getInstanceNum());
	  }
	}
	item = item->nextItem();
      }
    }
    type++;
  }

}

void CustomizeShortcutsDialogImpl::setShortcut(QKeySequence& sequence) {
    ListItem * listItem = dynamic_cast_listItem(List->currentItem());
    if (listItem) {
      QMap<int,int>::iterator it;
      it = keyMap.find(sequence);
      if ( it!=keyMap.end()) {

	QString top = "";
	bool done = FALSE;
	QListViewItem * topItem = List->firstChild();
	do {
	  QListViewItem * child = topItem->firstChild();
	  while (!done && child) {
	    if (child->text(1) == QString(sequence)) {
	      top = topItem->text(0) + " | ";
	      done = TRUE;
	    }
	    child = child->nextSibling();
	  }
	} while ( !done && (topItem=topItem->nextSibling()));

	QMessageBox::warning(this,tr("Can't use key shortcut"),
			     tr("This shortcut is already in use by:")
			     +"\n'"+top+MenuItem::getInstance(it.data())->caption()+"'",
			     QMessageBox::Cancel,QMessageBox::NoButton);

      } else {
	emit setText(QString(sequence));
	listItem->setText(1,QString(sequence));
	int instance = listItem->getMenuItem()->getInstanceNum();
	keyMap.remove(listItem->getAccel());
	if ((int)sequence != 0) {
	  keyMap.insert(sequence,instance);
	}
	listItem->setAccel(sequence);
      }
    }
}

void CustomizeShortcutsDialogImpl::accept() {
  applyClicked();
  CustomizeShortcutsDialog::accept();
}

void CustomizeShortcutsDialogImpl::enterWhatsThisMode()
{
   QWhatsThis::enterWhatsThisMode();
}
