/****************************************************************************
**
** $Id: PixmapTimer.cpp,v 1.10 2004/01/11 22:26:21 riemer Exp $
**
** Copyright (C) 2001-2004 Jose Hernandez <joseh@tesco.net>
**
**
**----------------------------------------------------------------------------
**
**----------------------------------------------------------------------------
**
** LinCVS is available under two different licenses:
**
** If LinCVS is linked against the GPLed version of Qt 
** LinCVS is released under the terms of GPL also.
**
** If LinCVS is linked against a nonGPLed version of Qt 
** LinCVS is released under the terms of the 
** LinCVS License for non-Unix platforms (LLNU)
**
**
** LinCVS License for non-Unix platforms (LLNU):
**
** Redistribution and use in binary form, without modification, 
** are permitted provided that the following conditions are met:
**
** 1. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 2. It is not permitted to distribute the binary package under a name
**    different than LinCVS.
** 3. The name of the authors may not be used to endorse or promote
**    products derived from this software without specific prior written
**    permission.
** 4. The source code is the creative property of the authors.
**    Extensions and development under the terms of the Gnu Public License
**    are limited to the Unix platform. Any distribution or compilation of 
**    the source code against libraries licensed other than gpl requires 
**    the written permission of the authors.
**
**
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR 
** IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
** ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY 
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
** GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
** WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
**
**
** LinCVS License for Unix platforms:
**
** This program is free software; you can redistribute it and/or modify 
** it under the terms of the GNU General Public License as published by 
** the Free Software Foundation; either version 2 of the License, or 
** (at your  option) any later version.  This program is distributed in 
** the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
** even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
** PARTICULAR PURPOSE.
**
** See the GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**
*****************************************************************************/


#include "config.h"

#include <qglobal.h>
#include <qtimer.h>
#include <qpixmap.h>
#include "PixmapTimer.h"
#include "pixmapcache.h"
#include "globals.h"



static const unsigned int CPrivXpmTimer_NUM_ANIM_PICS = 4;


struct CPrivXpmTimer
{
	//static const unsigned int NUM_ANIM_PICS = 4;

	QTimer *m_pTimer;
	const QPixmap *m_pIcon[CPrivXpmTimer_NUM_ANIM_PICS];   
	int m_curPic;

	// Counts number of activations.
	unsigned int m_nActive;
};


CPixmapTimer::CPixmapTimer(QObject * parent /*= 0*/, const char * name /*= 0*/)
	: QObject(parent, name)
{
	d = new CPrivXpmTimer;
	d->m_curPic = 0;
	d->m_nActive = 0;

	for (int i=0; i<(int)CPrivXpmTimer_NUM_ANIM_PICS; i++)
		d->m_pIcon[i] = &findEmbeddedPixmap(QString().sprintf("Connect32x16-%02d", i));

	d->m_pTimer = new QTimer(this);
	connect(d->m_pTimer, SIGNAL(timeout()), this, SLOT(slotProgress()));
}


CPixmapTimer::~CPixmapTimer()
{
	// The timer will automagically be deleted by Qt when CPixmapTimer is deleted.
	// The pixmaps will be deleted when the application exits as they are cached
	// in a dictionary and get reused.
	delete d;
}


void CPixmapTimer::slotStart()
{
	(d->m_nActive)++;

	if (!d->m_pTimer->isActive())
		d->m_pTimer->start(PROGRESSINTERVALL);
}


void CPixmapTimer::slotStop()
{
	if (d->m_nActive > 0)
		(d->m_nActive)--;

// 	if (d->m_nActive == 0)// don't need this since looped commands don't call slotStop more than once
//      but might call slotStart several times
	{
		if (d->m_pTimer->isActive())
			d->m_pTimer->stop();

		emit signalTimeout(*d->m_pIcon[d->m_curPic = 0]);
	}
}


void CPixmapTimer::slotProgress()
{
	++(d->m_curPic) %= CPrivXpmTimer_NUM_ANIM_PICS;
	emit signalTimeout(*d->m_pIcon[d->m_curPic]);
}
