/****************************************************************************
**
** $Id: cvslistview.h,v 1.30 2004/03/05 00:19:00 hemer Exp $
**
** Copyright (C) 2001-2004 The LinCVS development team.
**    Tilo Riemer <riemer@lincvs.org>
**    Falk Brettschneider <gigafalk@yahoo.com>
**    Frank Hemer <frank@hemer.org>
**    Wim Delvaux <wim.delvaux@chello.be>
**    Jose Hernandez <joseh@tesco.net>
**    Helmut Koll <HelmutKoll@web.de>
**    Tom Mishima <tmishima@mail.at-m.or.jp>
**    Joerg Preiss <auba@auba.de>
**    Sven Trogisch <trogisch@iapp.de>
**
**
**----------------------------------------------------------------------------
**
**----------------------------------------------------------------------------
**
** LinCVS is available under two different licenses:
**
** If LinCVS is linked against the GPLed version of Qt 
** LinCVS is released under the terms of GPL also.
**
** If LinCVS is linked against a nonGPLed version of Qt 
** LinCVS is released under the terms of the 
** LinCVS License for non-Unix platforms (LLNU)
**
**
** LinCVS License for non-Unix platforms (LLNU):
**
** Redistribution and use in binary form, without modification, 
** are permitted provided that the following conditions are met:
**
** 1. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 2. It is not permitted to distribute the binary package under a name
**    different than LinCVS.
** 3. The name of the authors may not be used to endorse or promote
**    products derived from this software without specific prior written
**    permission.
** 4. The source code is the creative property of the authors.
**    Extensions and development under the terms of the Gnu Public License
**    are limited to the Unix platform. Any distribution or compilation of 
**    the source code against libraries licensed other than gpl requires 
**    the written permission of the authors.
**
**
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR 
** IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
** ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY 
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
** GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
** WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
**
**
** LinCVS License for Unix platforms:
**
** This program is free software; you can redistribute it and/or modify 
** it under the terms of the GNU General Public License as published by 
** the Free Software Foundation; either version 2 of the License, or 
** (at your  option) any later version.  This program is distributed in 
** the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
** even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
** PARTICULAR PURPOSE.
**
** See the GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**
*****************************************************************************/


#ifndef CVSLISTVIEW_H_20010626
#define CVSLISTVIEW_H_20010626

//----------------------------------------------------------------------------

#include "config.h"

//----------------------------------------------------------------------------

#include <qlistview.h>
#include <qstringlist.h>
#include <qtimer.h>
#include <qptrlist.h>

//----------------------------------------------------------------------------

#include "FileListView.h"
#include "directory.h"

//----------------------------------------------------------------------------

class QPopupMenu;
class QListViewItem;

//----------------------------------------------------------------------------

class CvsListView: public CFileListView
{
   Q_OBJECT

public:
   CvsListView(QPopupMenu *menu, QWidget * parent = 0, const char * name = 0);

   FileListViewItem * firstChild() const;

protected:
   void viewportMousePressEvent(QMouseEvent * e);
   void keyPressEvent ( QKeyEvent * e );

   QPopupMenu *m_pMenu;
};

//----------------------------------------------------------------------------

class CvsDirListView: public QListView
{
   Q_OBJECT
public:
   CvsDirListView(QPopupMenu *menu, QWidget * parent = 0, const char * name = 0);

   Directory * find( QString);
   Directory * findBestMatch( QString);
   Directory * selectedItem() const;
   void setStatusBarText( const QString &, int ms=0);
   void paste(QStringList);

signals:
   void saveViewportPos();
   void recoverViewportPos();
   void typed();
   void showDirMenu( bool show);
   void setStatusText( const QString &, int);
   void folderDropped(const QString);
   void addCvsFiles(const QStringList,bool);
   void moveCvsFiles(const QString,const QStringList);

public slots:
   void selectionChanged();

protected:
   virtual void resizeEvent (QResizeEvent * e);
   void viewportMousePressEvent(QMouseEvent * e);
   void keyPressEvent ( QKeyEvent * e );
   void keyReleaseEvent ( QKeyEvent * e );
   void contentsMouseDoubleClickEvent ( QMouseEvent * e );
   void contentsMouseMoveEvent ( QMouseEvent * e );
   void contentsMousePressEvent ( QMouseEvent * e );
   void contentsMouseReleaseEvent ( QMouseEvent * e );
   virtual void dragEnterEvent(QDragEnterEvent *);
   virtual void dropEvent(QDropEvent *);
   virtual void dragMoveEvent ( QDragMoveEvent * );
   virtual void dragLeaveEvent ( QDragLeaveEvent * );

   QPopupMenu *m_pMenu;
   QPopupMenu * m_pDropMenu;
   QPopupMenu * m_pDropCvsMenu;
   Directory* OldItem;

 private:
   void popupMenu(Directory *, QPoint);
   void resetDrag();

   bool anySelected(QListViewItem *myChild);
   Directory * m_pDropDir;
   QStringList m_DropStringList;
   QTimer * m_openTimer;
   Directory * m_lastSelectedItem;
   QPtrList<Directory> m_openList;
   Directory * m_itemOnDragEnter;
   QTimer * m_scrollUpTimer;
   QTimer * m_scrollDownTimer;
   QTimer * m_scrollLeftTimer;
   QTimer * m_scrollRightTimer;

   int moveItem;
   int copyItem;
   int copyTreeItem;
   int addAsciiItem;
   int addBinaryItem;

   int cvsMoveItem;
   int cvsCopyItem;
   int cvsMoveCvsItem;
   int cvsAddAsciiItem;
   int cvsAddBinaryItem;

   int m_xScrollPos;
   int m_yScrollPos;

   int m_xDeltaAutoScroll;
   int m_yDeltaAutoScroll;

 private slots:

   void moveFiles();
   void copyFiles();
   void copyTree();
   void moveCvsFiles();
   void copyCvsFiles(bool);
   void copyCvsFilesAscii();
   void copyCvsFilesBinary();
   void openLastSelectedItem();

   void setScrollUpTimer(int);
   void setScrollDownTimer(int);
   void scrollUp();
   void scrollDown();
   void setScrollLeftTimer(int);
   void setScrollRightTimer(int);
   void scrollLeft();
   void scrollRight();
};

//----------------------------------------------------------------------------

#endif   //CVSLISTVIEW_H_20010626
