/****************************************************************************
**
** $Id: globals.cpp,v 1.132 2004/05/07 18:44:14 riemer Exp $
**
** Copyright (C) 1999-2004 Tilo Riemer <riemer@lincvs.org> and
**    Frank Hemer <frank@hemer.org>
**    Wim Delvaux <wim.delvaux@chello.be>
**    Jose Hernandez <joseh@tesco.net>
**    Helmut Koll <HelmutKoll@web.de>
**    Tom Mishima <tmishima@mail.at-m.or.jp>
**    Joerg Preiss <auba@auba.de>
**    Sven Trogisch <trogisch@iapp.de>
**
**
**----------------------------------------------------------------------------
**
**----------------------------------------------------------------------------
**
** LinCVS is available under two different licenses:
**
** If LinCVS is linked against the GPLed version of Qt 
** LinCVS is released under the terms of GPL also.
**
** If LinCVS is linked against a nonGPLed version of Qt 
** LinCVS is released under the terms of the 
** LinCVS License for non-Unix platforms (LLNU)
**
**
** LinCVS License for non-Unix platforms (LLNU):
**
** Redistribution and use in binary form, without modification, 
** are permitted provided that the following conditions are met:
**
** 1. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 2. It is not permitted to distribute the binary package under a name
**    different than LinCVS.
** 3. The name of the authors may not be used to endorse or promote
**    products derived from this software without specific prior written
**    permission.
** 4. The source code is the creative property of the authors.
**    Extensions and development under the terms of the Gnu Public License
**    are limited to the Unix platform. Any distribution or compilation of 
**    the source code against libraries licensed other than gpl requires 
**    the written permission of the authors.
**
**
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR 
** IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
** ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY 
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
** GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
** WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
**
**
** LinCVS License for Unix platforms:
**
** This program is free software; you can redistribute it and/or modify 
** it under the terms of the GNU General Public License as published by 
** the Free Software Foundation; either version 2 of the License, or 
** (at your  option) any later version.  This program is distributed in 
** the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
** even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
** PARTICULAR PURPOSE.
**
** See the GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**
*****************************************************************************/


#define DEFINE_GLOBALS

#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>

//----------------------------------------------------------------------------

#include <qfile.h>
#include <qdir.h>
#include <qtextstream.h>
#include <qdialog.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qobject.h> 
#include <qbutton.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include <qregexp.h>

#ifdef Q_WS_WIN
#include "win-globals.h"
#else
#if (defined __GLIBC__) || (defined Q_WS_MAC)
#define USE_TM_GMTOFF
#endif
#include <unistd.h>
#endif

//----------------------------------------------------------------------------

#include "time_ac.h"
#include "globals.h"
#include "wrapper.h"

#include "TextCodec.h"
#include "TextDecoder.h"
#include "TextEncoder.h"

#ifdef USE_FAM
#include "FamConnector.h"
#endif

#ifdef USE_WINNT_DIRMON
#include "WinNtConnector.h"
#endif

#ifdef USE_DNOTIFY
#include "DNotifyConnector.h"
#endif

//----------------------------------------------------------------------------

int sshPid;

//----------------------------------------------------------------------------

void initGlobals() {
  WINVERSION = 0;
#ifdef Q_WS_WIN
  WINVERSION = winitGlobals();
#endif
  determineTimeZoneDiff();
  projectSettings = new ProjectSettings();
}

//----------------------------------------------------------------------------

const QString CvsOptions::cmprStr()
{
  if (CvsOptions::g_compression > 0) {
    QString compressionStr = "-z%1";
    return compressionStr.arg(CvsOptions::g_compression);
  } else return "";
}

//----------------------------------------------------------------------------

#ifndef Q_WS_WIN
void determineTimeZoneDiff()
{
   timeZoneDiffInSecs = 0;
   struct tm *result = NULL;
   time_t t_loc = time(0);
   result = localtime(&t_loc);
   if (result) {//the difference to UTC without daylight diff
#ifdef USE_TM_GMTOFF
      timeZoneDiffInSecs = -(result->tm_gmtoff);
#else
      timeZoneDiffInSecs = timezone;
#endif
   }
   TIMEZERO.setTime_t(0);//ATTENTION: this is different from QDateTime()
}
#endif

//----------------------------------------------------------------------------


QDateTime getAsUTC(QDateTime &dateTime, bool hasDSTBug) {
  return convertTimeZone(dateTime, hasDSTBug, TRUE);
}

QDateTime getAsLocal(QDateTime &dateTime, bool hasDSTBug) {
  return convertTimeZone(dateTime, hasDSTBug, FALSE);
}

QDateTime convertTimeZone(QDateTime &dateTime, bool hasDSTBug, bool toUTC)
{
    //ATTENTION: we have to check the timezone for modification time
    //of every file because we have files modified in the winter
    //and files modified in the summer

#ifdef Q_WS_WIN
   bool f;
   QDateTime dati = cvsntConvertTimeZone(dateTime, hasDSTBug, toUTC, f);
   if (f) return dati;
#else
   Q_UNUSED(hasDSTBug)
#endif

#ifdef USE_TM_GMTOFF
    int diff = 0;
    struct tm *result = NULL;
    time_t t_loc = TIMEZERO.secsTo(dateTime);
    result = localtime(&t_loc);
    if (result) {
        diff = -(result->tm_gmtoff);
    }
#else
    int diff = timeZoneDiffInSecs;
    struct tm *result = NULL;

    time_t t_loc = TIMEZERO.secsTo(dateTime);
    result = localtime(&t_loc);
    if (result) {
        if (result->tm_isdst > 0) diff -= 3600;  //daylight is active for THIS file
    }
#endif
    if (toUTC) {
      return dateTime.addSecs(diff);
    } else {
      return dateTime.addSecs(-diff);
    }
}

//----------------------------------------------------------------------------

#ifndef Q_WS_WIN
bool hasDSTBug( QString*) {
   return false;
}
#endif

//----------------------------------------------------------------------------

#ifndef Q_WS_WIN
bool isInCvsPass(QString cvsRoot)
{
  QString method;
  QString user;
  QString passwd;
  QString host;
  int port;
  QString rootDir;
  extractCVSROOT( cvsRoot, method, user, passwd, host, port, rootDir);
  QString cvsRootNoPort = cvsRoot;
  if (port == 2401) {//only compare with portless root if default port
    cvsRootNoPort.replace( QRegExp(":"+QString::number(port)),":");//remove port to compare with entries
  } else {
    port = -1;
  }

  if (CVSVERSION == "cvs") {
    QFile f;
    QString line;

    //read .cvspass
    f.setName(CVSPASSPATH);
    if(f.open(IO_ReadOnly)) {//file is readable
      QTextStream textStream(&f); 
    
      while(!textStream.atEnd()) {
	line = textStream.readLine();
	line = line.replace( QRegExp(":"+QString::number(port)),":");//remove port to compare with entries
	if(line.find(cvsRootNoPort) != -1) {
	  f.close();
	  return true;
	}
      }
      f.close();
    }
  } else if (WINVERSION) {

    QString rootKey = "Software\\Cvsnt";
    QString subKey = "cvspass";

  }
  return false;
}
#endif

//----------------------------------------------------------------------------

#ifndef Q_WS_WIN
bool writeToCvsPassFile(const QString& cvsRoot, const QString& pwd) {

  if (CVSVERSION == "cvs") {
    QFile f;
    QString line = cvsRoot+" "+pwd+"\n";
    f.setName(CVSPASSPATH);
    if(f.open(IO_WriteOnly | IO_Append)) {
      QTextStream textStream(&f); 
      textStream << line;
      f.flush();
      f.close();
      chmod(CVSPASSPATH.ascii(),0600); // little paranoia always makes sense
      return true;
    }
  }
  return false;
}
#endif

//----------------------------------------------------------------------------

#ifndef Q_WS_WIN
bool removeFromCvsPassFile( QString cvsRoot) {

  QString method;
  QString user;
  QString passwd;
  QString host;
  int port;
  QString rootDir;
  extractCVSROOT( cvsRoot, method, user, passwd, host, port, rootDir);
  QString cvsRootNoPort = cvsRoot;
  cvsRootNoPort.replace( QRegExp(":"+QString::number(port)),":");//remove port to compare with entries

  if (CVSVERSION == "cvs") {

    QStringList list;
    QString line;
    QFile f;
    bool found = false;

    f.setName(CVSPASSPATH);
    if(f.open(IO_ReadOnly)) {
      QTextStream textStream(&f); 
      while(!textStream.atEnd()) {
	line = textStream.readLine();
	line = line.replace( QRegExp(":"+QString::number(port)),":");//remove port to compare with entries
	if (line.startsWith(cvsRootNoPort)) {
	  found = true;
	  continue;
	}
	list.append(line);
      }
      f.close();
      if (!found) {
	return false;
      }
      f.remove();
      if (f.open(IO_WriteOnly)) {
	QTextStream newTextStream(&f);
	for (QStringList::Iterator it = list.begin(); it != list.end(); ++it) {
	  newTextStream << (*it);
	  newTextStream << "\n";
	}
	f.flush();
	f.close();
	chmod(CVSPASSPATH.ascii(),0600); // little paranoia always makes sense
      }
    }
    return true;
  }
  return false;
}
#endif

//----------------------------------------------------------------------------

int extractCVSROOT( QString rootline,
		    QString& method,
		    QString& user,
		    QString& passwd,
		    QString& host,
		    int& port,
		    QString& path) {

  /* Access method specified, as in
   * "cvs -d :(gserver|kserver|pserver):[[user][:password]@]host[:[port]]/path",
   * "cvs -d [:(ext|server):][[user]@]host[:]/path",
   * "cvs -d :local:e:\path",
   * "cvs -d :fork:/path".
   *
   * Note that path for connecting to a cvs-winnt server might be something like:
   * "/d//somepath/cvs"
   */

  int from, to;
  int atpos;
  QString tmpStr;

  //method
  if (rootline.at(0) == ':') {
    from = 1;
    to = rootline.find(':',from);
    if (to == -1) return false;//syntax error
    method = rootline.mid((unsigned int)from,(unsigned int)(to-from));
    rootline = rootline.mid((unsigned int)(to+1));
  } else if ( (rootline.at((unsigned int)0) != '/') && (rootline.contains('/')) ) {
    method = "ext";
  } else {
    method = "local";
  }

  if ( (method == "local") || (method == "fork")) {
    if ( rootline.find(':') > 1) return false;
    path = rootline;
    user = QString::null;
    passwd = QString::null;
    host = QString::null;
    port = -1;
    return true;
  }

  //path
  from = rootline.find('/');
  if (from >= 0) {//cvs server running on UNIX
    path = rootline.mid((unsigned int)from);
    rootline.truncate((unsigned int)from);
  } else {//cvsnt server running on winXX
    from = rootline.find('\\');
    if (from > 2) {//path found. it can contain a drive letter ... I love win ;-)
      if (rootline[from-2].isLetter() &&
          (rootline[from-1] == ':') &&
          (rootline[from-3] == ':')) from -= 2;

      path = rootline.mid((unsigned int)from);
      rootline.truncate((unsigned int)from);
    } else return false;
  }


  //user:password@
  atpos = rootline.find('@');
  if (atpos >= 0) {
    to = rootline.findRev(':',atpos-1-rootline.length());
    if (to >= 0) {
      user = rootline.mid((unsigned int)0,(unsigned int)to);
      passwd = rootline.mid((unsigned int)(to+1),(unsigned int)(atpos-to-1));
    } else {
      user = rootline.mid((unsigned int)0,(unsigned int)atpos);
      passwd = QString::null;
    }
    if (user.isEmpty()) return false;//no user
  } else {
    user = QString::null;
    passwd = QString::null;
  }
  rootline = rootline.mid((unsigned int)(atpos+1));


  //host:port
  from = rootline.find(':');
  to = rootline.find(':',from+1);
  if (from == -1) {
    host = rootline;
    port = 2401;
  } else if (to == -1) {
    tmpStr = rootline.mid((unsigned int)(from+1));
    if (tmpStr.isEmpty()) {
      port = 2401;
    } else {
      port = tmpStr.toInt();
    }
  } else {
    port = rootline.mid((unsigned int)(from+1),(unsigned int)(to-from-1)).toInt();
  }
  host = rootline.mid((unsigned int)0,(unsigned int)from);


  if (host.isEmpty()) return false;//no host
  if (port == 0) return false;//no number ie. syntax error
  return true;
}

//----------------------------------------------------------------------------

int analyzeCVSROOT( QString& method,
		    QString& user,
		    QString& passwd,
		    QString& host,
		    int& port,
		    QString& path) {

  QString line = getenv("CVSROOT");
  if (!line.isEmpty()) {
    return extractCVSROOT(line, method, user, passwd, host, port, path);
  } else {
    return false;
  }
}

//----------------------------------------------------------------------------

void removeDoubleSlashes(QString& s)
{
   int pos;
   while ( (pos = s.find("//")) > -1) s.replace(pos, 2, "/");
}

//----------------------------------------------------------------------------
#ifndef Q_WS_WIN
void startSshAgent() {

  bool useAgent = true;
  if (bUseSshAgentVars) {
    QString envValue;
    if ( envValue=getenv("SSH_AUTH_SOCK")) {
      envSSH_AUTH_SOCK = "SSH_AUTH_SOCK=" + envValue;
    } else if ( envValue=getenv("SSH2_AUTH_SOCK")) {
      envSSH_AUTH_SOCK = "SSH2_AUTH_SOCK=" + envValue;
    } else {
      useAgent = false;
    }
    if ( envValue=getenv("SSH_AGENT_PID")) {
      envSSH_AGENT_PID = "SSH_AGENT_PID=" + envValue;
    } else if ( envValue=getenv("SSH2_AGENT_PID")) {
      envSSH_AGENT_PID = "SSH2_AGENT_PID=" + envValue;
    } else {
      useAgent = false;
    }
    if (useAgent) {
      sshAgentStarted = true;
      sshAgentIsRunning = true;
      sshPid = -1;//don't kill agent on exit
      return;
    }
  }

  QString sh = APPDIR + "/" + LC_TOOLS_DIR + "/start-ssh-agent.sh";
  QString askpass = APPDIR + "/" + LC_TOOLS_DIR + "/ssh-askpass.bin";

  QStringList * env = NULL;
  sshProc = new QProcess(sh);

  if (QFileInfo(askpass).exists()) {
    env = new QStringList();
    QString tmpEnv;
    QString tmpVar;
    tmpVar = "DISPLAY";
    tmpEnv = getenv(tmpVar);
    if (tmpEnv.length() != 0) {
      env->append(tmpVar+"="+tmpEnv);
    }
    tmpVar = "PATH";
    tmpEnv = getenv(tmpVar);
    if (tmpEnv.length() != 0) {
      env->append(tmpVar+"="+tmpEnv);
    }
    tmpVar = "HOME";
    tmpEnv = getenv(tmpVar);
    if (tmpEnv.length() != 0) {
      env->append(tmpVar+"="+tmpEnv);
    }
    env->append("SSH_ASKPASS="+askpass);
  }

  if (sshProc->start(env)) {
    while (sshProc->isRunning()) {
      qApp->processEvents();
      if (globalStopAction) sshProc->kill();
    }
    if ( (sshProc->normalExit()) && (sshProc->exitStatus() == 0) ) {
      sshAgentStarted = true;
    } else {
      checkSshAgent();
      killSshAgent();
    }
  } else {
    qDebug("couldn't start proc: ->"+sh+"<-");
  }
  delete sshProc;
}

//----------------------------------------------------------------------------

void checkSshAgent() {

  QFile f;
  QString line;

  //now read the environment file and ascertain the PID of ssh-agent

  f.setName(QDir::homeDirPath() + "/.ssh/lincvs-environment");
   
  sshPid = -1;
  sshAuthSock = "";
  
  if(f.open(IO_ReadOnly)) {//file is readable
    QTextStream textStream(&f); 
    
    while(!textStream.atEnd()) {
      line = textStream.readLine();
      if (line.find("setenv SSH_AUTH_SOCK ") == 0) {
	QString tmp = line.mid(21);
	envSSH_AUTH_SOCK = "SSH_AUTH_SOCK=" + tmp.left(tmp.find(";"));
      } else if(line.contains("_AUTH_SOCK=") > 0) {//SSH or SSH2
	envSSH_AUTH_SOCK = line.left(line.find(";"));
      }
      if (line.find("setenv SSH_AGENT_PID ") == 0) {
	QString tmp = line.mid(21, 20);
	QChar ch;
	ch = tmp[0];
	unsigned int ch_pos = 0;
	while (ch.isDigit() && (tmp.length() > ch_pos)) {
	  ch_pos++;
	  ch = tmp.at(ch_pos);
	}
	QString pid = tmp.left(ch_pos);
	envSSH_AGENT_PID = "SSH_AGENT_PID="+pid;
	sshPid = pid.toInt();
      } else if(line.contains("_AGENT_PID=") > 0) {//SSH or SSH2
	envSSH_AGENT_PID = line.left(line.find(";"));
	QString pid = envSSH_AGENT_PID.mid(envSSH_AGENT_PID.find("=")+1);
	sshPid = pid.toInt();
      }
    }
  }   
  
  if (sshPid > 0 && (!envSSH_AUTH_SOCK.isEmpty())) sshAgentIsRunning = true;
  else sshAgentIsRunning = false;  //sshPid not found, should never happen
}

//----------------------------------------------------------------------------

void killSshAgent() {
    if (sshPid > 0) {
        QString pid;
        pid.setNum(sshPid);
        system("kill " + pid);
	sshAgentIsRunning = false;
	sshAgentStarted = false;
	sshPid = -1;
    }
}
#endif
//----------------------------------------------------------------------------

QString writeTmpFile(QString text)
{
   QString fname = createTempFile();
   if (fname.length()) {
      QFile file(fname);
      if (file.open(IO_WriteOnly)) {
	QTextStream s(&file);                        // serialize using f
        s.setCodec(I18n::g_pTextEncoder->pTextCodecPool()->pCodecForName(I18n::g_nameOfEncoder));
        s << text;     // serialize string
      }
      file.close();
   }
   
   return fname;
}

//----------------------------------------------------------------------------

void writeUserHstFile(QString text)
{
   QString fname = createHistoryFile();

   if (fname.length()) {
      QFile file(fname);
      if (file.open(IO_WriteOnly)) {                  // open file for writing
        QTextStream s(&file);                        // serialize using f
        s.setCodec(I18n::g_pTextEncoder->pTextCodecPool()->pCodecForName(I18n::g_nameOfEncoder));
        s << text;     // serialize string
      }
      file.close();
   }
}

//----------------------------------------------------------------------------

QString readTextFile(QString fn)
{
    QFile f(fn);
    QString str = "";

    if (f.open(IO_ReadOnly)) {
        QTextStream ts(&f);
        ts.setCodec(I18n::g_pTextDecoder->pTextCodecPool()->pCodecForName(I18n::g_nameOfDecoder));
        QString line;
        while(!ts.eof()) {
           line = ts.readLine();
           str += line + "\n";
        }
        f.close();
    }

    return str;
}

//----------------------------------------------------------------------------

QString readDefaultTemplateFile()
{
    return readTextFile(tmpDir + "/Template/rcsinfo");
}

//----------------------------------------------------------------------------

void writeDefaultTemplateFile(QString text)
{
    QFile rcsFile(tmpDir + "/Template/rcsinfo");

    if (text.isEmpty()) {//no text for template --> remove rcsinfo
        if (rcsFile.exists()) rcsFile.remove();
    } else {//we have some text --> write it into rcsinfo
        createTemplateDir();  //create template dir if necessary

        if (rcsFile.open(IO_WriteOnly)) {
           QTextStream s(&rcsFile);                     // serialize using f
           s.setCodec(I18n::g_pTextEncoder->pTextCodecPool()->pCodecForName(I18n::g_nameOfEncoder));
           s << text;     // serialize string
        }
        rcsFile.close();
    }

}
 
//----------------------------------------------------------------------------

void createTmpDir()
{
  QDir home = QDir::home();
  QDir tempDir;

  QFileInfo tmp(home, ".lincvs");
  if (tmp.exists()) {
    if (!tmp.isDir()) {
      QFile f(tmp.absFilePath());
      f.remove();  //we must check the result of remove!!!
      tempDir.mkdir(tmp.absFilePath());
    }
  } else tempDir.mkdir(tmp.absFilePath());
 
#ifndef Q_WS_WIN
  chmod(tmp.absFilePath(), 0700);
#endif

  tmpDir = tmp.absFilePath();
}

//----------------------------------------------------------------------------

#ifndef Q_WS_WIN
QString createTempFile(QString appendix /*= QString::null*/)
{
  if (!appendix.isNull()) appendix.replace(QRegExp("[\\/<>&]"),"_");
    
  QString tmp = tmpDir + "/tmp.XXXXXX";

  char* fname = new char[tmp.length() + 1];
  strcpy(fname, tmp);

  int res = mkstemp(fname);

  if (res == -1) {
    qDebug("Can't create tempfile: "+QString(fname));
    delete [] fname;
    return "";
  } else if (remove(fname) == -1) {//remove the file because: 1. we only need its name 2. this should be handled as on the win platform
    qDebug("Can't remove tempfile: "+QString(fname));
    delete [] fname;
    return "";
  }
  
  QString tmpFile = fname + appendix;
  delete [] fname;

  if (QFileInfo(tmpFile).exists()) {
    qDebug("tempfile exists: "+tmpFile);
    return "";
  }
  return tmpFile;
}
#endif

//----------------------------------------------------------------------------

void createTemplateDir()
{
    QDir tempDir;
    QFileInfo tmp(tmpDir, "Template");
    if (tmp.exists()) {
        if (!tmp.isDir()) {
            QFile f(tmp.absFilePath());
            f.remove();  //we must check the result of remove!!!
            tempDir.mkdir(tmp.absFilePath());
        }
    } else tempDir.mkdir(tmp.absFilePath());

#ifndef Q_WS_WIN
    chmod(tmp.absFilePath(), 0700);
#endif
}

//----------------------------------------------------------------------------

#ifndef Q_WS_WIN
QString createHistoryFile()
{
  createTemplateDir();

  QString tmpl = tmpDir + "/Template/XXXXXX";

  char* fname = new char[tmpl.length() + 1];
  strcpy(fname, tmpl);
   
  int file = mkstemp(fname);
  
  QString tmpFile = fname;
  delete [] fname;      

  if (file > -1) {
    fchmod(file, 0600);
    close(file);

    return tmpFile;
  }
  
  return "";
}
#endif

//----------------------------------------------------------------------------

void runExternal( QString arg, QDir *dir) {

  arg = LW::validateEchse(arg);
  QString debugStr = "running: "+arg+" in: ";
  if (dir) debugStr += dir->absPath();
  else debugStr += "default dir";
  qDebug(debugStr);
  arg += " ";

  int S = 1;
  int K = 2;
  int blocked = 0;
  unsigned int i;
  unsigned int count = 0;
  unsigned int lastPos = 0;
  QStringList args;

  for (i=0; i<arg.length(); i++) {
    if ( arg.at(i) == '\"') {
      if (blocked != S) {
	count++;
	blocked = S;
      } else {
	count--;
	blocked = K;
      }
    }
    if ( arg.mid(i,1) == "'" ) {
      if (blocked != K) {
	count++;
	blocked = K;
      } else {
	count--;
	blocked = S;
      }
    }
    if ( (count == 0) && (arg.at(i) == ' ')) {
      QString tmp = arg.mid(lastPos,i-lastPos);
      if ( (tmp.startsWith("\"")) || (tmp.startsWith("'")) ) {
	tmp = tmp.mid(1);
      }
      if ( (tmp.endsWith("\"")) || (tmp.endsWith("'")) ) {
	tmp = tmp.left(tmp.length()-1);
      }
      if (!tmp.isEmpty()) args.append (tmp);
      lastPos = i+1;
    }
  }
//   for ( QStringList::Iterator it = args.begin(); it != args.end(); ++it ) {
//     qDebug("cmd: ->"+(*it)+"<-");
//   }
  runExternal( args, dir);
}

//----------------------------------------------------------------------------

void runExternal( QStringList args, QDir *dir) {

#ifdef Q_WS_WIN
  for ( QStringList::Iterator it = args.begin(); it != args.end(); ++it ) {
    if ( (*it).at(1) == ':') (*it).replace( QRegExp("/"),"\\");//convert separators if string seems to be a win path
  }
#endif

  QProcess proc;
  if (dir) {
    proc.setWorkingDirectory(*dir);
  }
  proc.setArguments(args);
  if (!proc.start()) {
    QString msg;
    if (dir) msg += "in dir: ->"+dir->absPath()+"<-, ";
    msg += "couldn't start proc: ->"+args.join(" ")+"<-";
    qDebug(msg);
  }
}

//----------------------------------------------------------------------------

void adaptQStringList(const QString &s, QStringList *list, int size) {
    if (s.isEmpty()) {
      return;
    }
    if (list->findIndex(s) == -1) {
	//not in list
	while ( (list->count() >= (unsigned int)size) && (list->count() > 0)) {
	    list->remove(--(list->end()));//remove the last element
	}
	list->prepend(s);
    } else if (list->findIndex(s) > 0) {
	// text isn't the first item in list
	list->remove(s);
	list->prepend(s);
    }
}

//----------------------------------------------------------------------------

void simplifyStringList(QStringList* pList)
{
    QStringList::Iterator it, head;
    bool bAllDone = false;

    head = pList->begin();

    while(!bAllDone) {
        it = head;
        if (it == pList->end()) {
            bAllDone = true;
//             qDebug("DONE");
        } else {
            //qDebug(*head);   //the lovely win doesn't like the star before head
            QString cur = *it;
            it++;   //we need the second item

            while(it != pList->end()) {
                if (*it == cur) {
//                     qDebug("remove: " + *it);
                    //remove the multiple entry
                    it = pList->remove(it);
                } else it++;
            }

            head++;
        }
    }

    // remove empty entries
    it = pList->begin();
    while(it != pList->end()) {
        if ((*it).isEmpty()) it = pList->remove(it);
        else it++;
    }
    
#ifdef ANY_DEBUG_STUFF
    qDebug("StringList: Start");
    for (it = pList->begin(); it != pList->end(); it++) {
        qDebug(*it);
    }
    qDebug("simplify StringList: End");
#endif
}

//----------------------------------------------------------------------------

void cutList(QStringList* pList, int allowedSize)
{
    int size = pList->count();
    int diff = size - allowedSize;
    
    if (diff < 1) {
        return;
    }

    int i;
    for (i = 0; i < diff; i++) {
        pList->remove(pList->at(size - 1 - i));
    }
}

//----------------------------------------------------------------------------

QString masqWs(const QString &str) {
  return "\""+str+"\"";
}

//----------------------------------------------------------------------------

QString umasqWs(QString str) {
  str = str.stripWhiteSpace();
  if (str.startsWith("\"")) {
    str.remove(0,1);
  }
  if (str.endsWith("\"")) {
    str.truncate(str.length()-1);
  }
  return str;
}

//----------------------------------------------------------------------------

bool masqedFilenamesForPlaceholders(QString& params, const QStringList& nameList) {

  unsigned int oldSize = params.length();
  int pos = 0;
  while ( (pos = params.find("%",pos)) > -1) {
    ++pos;
    unsigned int i = 0;
    while ( (params.length() > ((unsigned)pos)+i) && (params.at(((unsigned)pos)+i).isDigit()) ) i++;
    if (i>0) {
      QString exp = params.mid(pos,i);
      int namePos = exp.toInt()-1;
      if ( (namePos > -1) && (nameList.count() > (unsigned)namePos) ) {
	params.replace(QRegExp("%"+params.mid(pos,i)),"\""+nameList[namePos]+"\"");
      }
    }
  }
  QString allFiles = nameList.join("\" \"");
  params.replace(QRegExp("%n"),"\""+allFiles+"\"");

  if (oldSize<params.length()) return TRUE;
  else return FALSE;
}

//----------------------------------------------------------------------------

#ifndef Q_WS_WIN
bool setPermission(const QFile& file, int perm) {

  if (perm <= 7) {
    perm = perm << 6;
    struct stat buf;
    if (!stat(file.name().latin1(),&buf)) {//only change owner permissions
      mode_t mode = buf.st_mode & (S_IRWXG | S_IRWXO);
      perm += mode;
    } else {
      return false;
    }
  }
  if (!chmod( file.name().latin1(),perm)) {
    return true;
  }
  return false;
}
#endif

//----------------------------------------------------------------------------

#ifndef Q_WS_WIN
bool rename(const QString& src, const QString& dst) {
  QFile f(src);
  bool readonly = false;
  if (!QFileInfo(f).isWritable()) {
    if (!setPermission(f,READABLE | WRITEABLE)) {
      return false;
    }
    readonly = true;
  }
  if (rename(src.latin1(),dst.latin1())) {
    return false;
  }
  if (readonly) {
    f.setName(dst);
    if (!setPermission(f,READABLE)) {
      return false;
    }
  }
  return true;
}
#endif

//----------------------------------------------------------------------------

//return proxystring if available
//settings should be entered like:
//user@my.remote.server::otheruser@my.proxy.server
//where user and otheruser are optional
QString getProxy(const QString& userAtHost) {

  QString host = userAtHost.mid(userAtHost.find("@")+1);

  QString proxy;

  for (QStringList::Iterator proxyit = proxyList.begin(); proxyit != proxyList.end(); proxyit++) {
    if ( (*proxyit).startsWith(userAtHost)) {
      proxy = (*proxyit);
      break;
    }
  }
  if (proxy.isEmpty()) {//no proxy found, try again without user spec
    for (QStringList::Iterator proxyit = proxyList.begin(); proxyit != proxyList.end(); proxyit++) {
      if ( (*proxyit).startsWith(host)) {
	proxy = (*proxyit);
	break;
      }
    }
  }
  if (!proxy.isEmpty()) {
    int splitPos;
    if ( (splitPos=proxy.find("::")) >= 0) {
      proxy = proxy.mid(splitPos+2);
      int etpos;
      if ( (etpos=proxy.find("@"))>=0) {
	proxy = proxy.mid(etpos+1);
      }
      return proxy.stripWhiteSpace();
    }
  }
  return "";
}

//----------------------------------------------------------------------------

#ifndef Q_WS_WIN
void wait( int msecs) {
  struct timespec req;
  req.tv_sec = 0;
  req.tv_nsec = msecs*1000;
  nanosleep(&req,NULL);
}
#endif

//----------------------------------------------------------------------------

QString getSystemLF() {
#ifdef Q_WS_WIN
  return "\r\n";
#else
  return "\n";
#endif
}

//----------------------------------------------------------------------------

void I18n::init()
{
  CTextCodecPool* pTextCodecPool = new CTextCodecPool();
  
  I18n::g_pTextDecoder = new CMetaTextDecoder(pTextCodecPool);
  I18n::g_pTextDecoder->setGeneralDecoder(I18n::g_nameOfDecoder);

  I18n::g_pTextEncoder = new CTextEncoder(pTextCodecPool);
  I18n::g_pTextEncoder->setEncoder(I18n::g_nameOfEncoder);
}

//----------------------------------------------------------------------------

void I18n::destroy()
{
  CTextCodecPool* pTextCodecPool = I18n::g_pTextDecoder->pTextCodecPool();
  delete I18n::g_pTextDecoder;
  delete I18n::g_pTextEncoder;
  delete pTextCodecPool;
}

//----------------------------------------------------------------------------

bool initMonitoring() {
  fc = NULL;
  m_hasDirWatch = FALSE;
  DirWatch::b_isActive = FALSE;

#ifdef USE_FAM
  m_hasDirWatch = true;
  if ( DirWatch::b_useDirWatch) {
    if ((fc = new FamConnector()) && fc->isRunning()) {
      DirWatch::b_isActive = true;
    } else {
      qDebug("init fam failed");
    }
  }
#endif
  
#ifdef USE_WINNT_DIRMON
  m_hasDirWatch = true;
  if ( DirWatch::b_useDirWatch) {
    if ((fc = new CWinNtConnector()) && fc->isRunning()) {
      DirWatch::b_isActive = true;
    } else {
      qDebug("init directory monitoring failed");
    }
  }
#endif

#ifdef USE_DNOTIFY
  m_hasDirWatch = true;
  if ( DirWatch::b_useDirWatch) {
    if ((fc = new DNotifyConnector()) && fc->isRunning()) {
      DirWatch::b_isActive = true;
    } else {
      qDebug("init dnotify failed");
    }
  }
#endif

  return DirWatch::b_isActive;
}

DirConnector* getDirConnector() {
  return fc;
}

bool hasDirWatch() {
  return m_hasDirWatch;
}

void releaseMonitoring() {
  if (fc) {
    DirWatch::b_isActive = false;
    delete fc;
    fc = NULL;
  }
}

//----------------------------------------------------------------------------

bool addMonitoredDir(Directory * dir) {
  if (!DirWatch::b_isActive) return FALSE;
  bool added = fc->addMonitoredDir( dir);
  if (!added) {
    releaseMonitoring();   //hope it works here
  }
  return added;
}

//----------------------------------------------------------------------------

void releaseMonitoredDir(Directory * dir) {
  if (!DirWatch::b_isActive) return;
  fc->releaseMonitoredDir( dir);
  return;
}

//----------------------------------------------------------------------------

Directory* checkMonitoredEvent(QString& fileName,bool& deleted) {
  if (!DirWatch::b_isActive) return NULL;
  return fc->checkEvent(fileName,deleted);
}

void suspendDirWatch() {
  if (!DirWatch::b_isActive) return;
  fc->suspend();
}

void resumeDirWatch() {
  if (!DirWatch::b_isActive) return;
  fc->resume();
}

void lockDirWatch() {
  if (!DirWatch::b_isActive) return;
  fc->lock();
}

void unlockDirWatch() {
  if (!DirWatch::b_isActive) return;
  fc->unlock();
}
